@@ -150,9 +150,9 @@ package Attean::Algebra::LeftJoin 0.009 {
 		
 		my @tokens;
 		push(@tokens, $l);
-		push(@tokens, $lhs->sparql_tokens->members);
+		push(@tokens, $lhs->sparql_tokens->elements);
 		push(@tokens, $r, $opt, $l);
-		push(@tokens, $rhs->sparql_tokens->members);
+		push(@tokens, $rhs->sparql_tokens->elements);
 		push(@tokens, $r);
 		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
 	}
@@ -226,9 +226,9 @@ package Attean::Algebra::Union 0.009 {
 		
 		my @tokens;
 		push(@tokens, $l);
-		push(@tokens, $lhs->sparql_tokens->members);
+		push(@tokens, $lhs->sparql_tokens->elements);
 		push(@tokens, $r, $union, $l);
-		push(@tokens, $rhs->sparql_tokens->members);
+		push(@tokens, $rhs->sparql_tokens->elements);
 		push(@tokens, $r);
 		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
 	}
@@ -256,7 +256,7 @@ package Attean::Algebra::Graph 0.009 {
 		my ($child)	= @{ $self->children };
 		my @vars	= $child->in_scope_variables;
 		if ($graph->does('Attean::API::Variable')) {
-			return Set::Scalar->new(@vars, $graph->value)->members;
+			return Set::Scalar->new(@vars, $graph->value)->elements;
 		} else {
 			return @vars;
 		}
@@ -289,9 +289,9 @@ package Attean::Algebra::Graph 0.009 {
 		
 		my @tokens;
 		push(@tokens, $graph);
-		push(@tokens, $self->graph->sparql_tokens->members);
+		push(@tokens, $self->graph->sparql_tokens->elements);
 		push(@tokens, $l);
-		push(@tokens, $child->sparql_tokens->members);
+		push(@tokens, $child->sparql_tokens->elements);
 		push(@tokens, $r);
 		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
 	}
@@ -308,7 +308,7 @@ package Attean::Algebra::Extend 0.009 {
 		my $self	= shift;
 		my ($child)	= @{ $self->children };
 		my @vars	= $child->in_scope_variables;
-		return Set::Scalar->new(@vars, $self->variable->value)->members;
+		return Set::Scalar->new(@vars, $self->variable->value)->elements;
 	}
 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
 	has 'variable' => (is => 'ro', isa => ConsumerOf['Attean::API::Variable'], required => 1);
@@ -396,9 +396,9 @@ package Attean::Algebra::Minus 0.009 {
 		
 		my @tokens;
 		push(@tokens, $l);
-		push(@tokens, $lhs->sparql_tokens->members);
+		push(@tokens, $lhs->sparql_tokens->elements);
 		push(@tokens, $r, $minus, $l);
-		push(@tokens, $rhs->sparql_tokens->members);
+		push(@tokens, $rhs->sparql_tokens->elements);
 		push(@tokens, $r);
 		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
 	}
@@ -414,7 +414,7 @@ package Attean::Algebra::Distinct 0.009 {
 	use namespace::clean;
 
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
-	with 'Attean::API::SPARQLSerializable';
+	with 'Attean::API::SPARQLQuerySerializable';
 
 	sub algebra_as_string { return 'Distinct' }
 	sub as_sparql {
@@ -433,11 +433,6 @@ package Attean::Algebra::Distinct 0.009 {
 				. "${indent}}\n";
 		}
 	}
-
-	sub sparql_tokens {
-		my $self	= shift;
-		return $self->query_tokens;
-	}
 }
 
 =item * L<Attean::Algebra::Reduced>
@@ -449,7 +444,7 @@ package Attean::Algebra::Reduced 0.009 {
 	use namespace::clean;
 
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
-	with 'Attean::API::SPARQLSerializable';
+	with 'Attean::API::SPARQLQuerySerializable';
 
 	sub algebra_as_string { return 'Reduced' }
 	sub as_sparql {
@@ -468,11 +463,6 @@ package Attean::Algebra::Reduced 0.009 {
 				. "${indent}}\n";
 		}
 	}
-
-	sub sparql_tokens {
-		my $self	= shift;
-		return $self->query_tokens;
-	}
 }
 
 =item * L<Attean::Algebra::Slice>
@@ -485,7 +475,7 @@ package Attean::Algebra::Slice 0.009 {
 	use namespace::clean;
 
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
-	with 'Attean::API::SPARQLSerializable';
+	with 'Attean::API::SPARQLQuerySerializable';
 
 	has 'limit' => (is => 'ro', isa => Int, default => -1);
 	has 'offset' => (is => 'ro', isa => Int, default => 0);
@@ -519,11 +509,6 @@ package Attean::Algebra::Slice 0.009 {
 		$sparql	.= "${indent}OFFSET " . $self->offset . "\n" if ($self->offset > 0);
 		return $sparql;
 	}
-
-	sub sparql_tokens {
-		my $self	= shift;
-		return $self->query_tokens;
-	}
 }
 
 =item * L<Attean::Algebra::Project>
@@ -536,7 +521,7 @@ package Attean::Algebra::Project 0.009 {
 	use namespace::clean;
 
 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
-	with 'Attean::API::SPARQLSerializable';
+	with 'Attean::API::SPARQLQuerySerializable';
 
 	has 'variables' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Variable']], required => 1);
 
@@ -545,7 +530,7 @@ package Attean::Algebra::Project 0.009 {
 		my ($child)	= @{ $self->children };
 		my $set		= Set::Scalar->new( $child->in_scope_variables );
 		my $proj	= Set::Scalar->new( map { $_->value } @{ $self->variables } );
-		return $set->intersection($proj)->members;
+		return $set->intersection($proj)->elements;
 	}
 	sub algebra_as_string {
 		my $self	= shift;
@@ -588,11 +573,6 @@ package Attean::Algebra::Project 0.009 {
 		
 		return $sparql;
 	}
-
-	sub sparql_tokens {
-		my $self	= shift;
-		return $self->query_tokens;
-	}
 }
 
 =item * L<Attean::Algebra::Comparator>
@@ -601,9 +581,16 @@ package Attean::Algebra::Project 0.009 {
 
 package Attean::Algebra::Comparator 0.009 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(Bool ConsumerOf);
+	use namespace::clean;
+
+	with 'Attean::API::SPARQLSerializable';
+
 	has 'ascending' => (is => 'ro', isa => Bool, default => 1);
 	has 'expression' => (is => 'ro', isa => ConsumerOf['Attean::API::Expression'], required => 1);
+
 	sub tree_attributes { return qw(expression) };
 	sub as_string {
 		my $self	= shift;
@@ -621,6 +608,24 @@ package Attean::Algebra::Comparator 0.009 {
 			return 'DESC(' . $self->expression->as_sparql . ')';
 		}
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $asc		= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['ASC'] );
+		my $desc	= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['DESC'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LPAREN, -1, -1, -1, -1, ['('] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RPAREN, -1, -1, -1, -1, [')'] );
+		
+		my @tokens;
+		if ($self->ascending) {
+			push(@tokens, $self->expression->sparql_tokens->elements);
+		} else {
+			push(@tokens, $desc, $l);
+			push(@tokens, $self->expression->sparql_tokens->elements);
+			push(@tokens, $r);
+		}
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::OrderBy>
@@ -629,9 +634,16 @@ package Attean::Algebra::Comparator 0.009 {
 
 package Attean::Algebra::OrderBy 0.009 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ArrayRef InstanceOf);
+	use namespace::clean;
+	
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+	
 	has 'comparators' => (is => 'ro', isa => ArrayRef[InstanceOf['Attean::Algebra::Comparator']], required => 1);
+	
 	sub tree_attributes { return qw(comparators) };
 	sub algebra_as_string {
 		my $self	= shift;
@@ -677,7 +689,7 @@ package Attean::Algebra::BGP 0.009 {
 			my @vars	= map { $_->value } grep { $_->does('Attean::API::Variable') } $t->values;
 			$set->insert(@vars);
 		}
-		return $set->members;
+		return $set->elements;
 	}
 	
 	sub sparql_tokens {
@@ -740,13 +752,16 @@ package Attean::Algebra::BGP 0.009 {
 =cut
 
 package Attean::Algebra::Service 0.009 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Moo;
 	use Types::Standard qw(ConsumerOf Bool);
 	use namespace::clean;
 
 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree', 'Attean::API::UnionScopeVariables';
-	has 'endpoint' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
+	with 'Attean::API::SPARQLSerializable';
 
+	has 'endpoint' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
 	has 'silent' => (is => 'ro', isa => Bool, default => 0);
 	
 	sub algebra_as_string {
@@ -768,6 +783,22 @@ package Attean::Algebra::Service 0.009 {
 			. $child->as_sparql( %args, level => $level+1 )
 			. "${indent}}\n";
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $service	= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['SERVICE'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] );
+		my ($child)	= @{ $self->children };
+		
+		my @tokens;
+		push(@tokens, $service);
+		push(@tokens, $self->endpoint->sparql_tokens->elements);
+		push(@tokens, $l);
+		push(@tokens, $child->sparql_tokens->elements);
+		push(@tokens, $r);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::Path>
@@ -791,7 +822,7 @@ package Attean::Algebra::Path 0.009 {
 	sub in_scope_variables {
 		my $self	= shift;
 		my @vars	= map { $_->value } grep { $_->does('Attean::API::Variable') } ($self->subject, $self->object);
-		return Set::Scalar->new(@vars)->members;
+		return Set::Scalar->new(@vars)->elements;
 	}
 
 	sub tree_attributes { return qw(subject path object) };
@@ -835,7 +866,16 @@ package Attean::Algebra::Path 0.009 {
 package Attean::Algebra::Group 0.009 {
 	use utf8;
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ArrayRef ConsumerOf);
+	use namespace::clean;
+	
+	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+
+	has 'groupby' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Expression']]);
+	has 'aggregates' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::AggregateExpression']]);
 	
 	sub BUILD {
 		my $self	= shift;
@@ -872,9 +912,7 @@ package Attean::Algebra::Group 0.009 {
 		}
 		return sprintf('Group { %s } aggregate { %s }', join(', ', map { $_->as_sparql() } @$groups), join(', ', @aggs));
 	}
-	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
-	has 'groupby' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Expression']]);
-	has 'aggregates' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::AggregateExpression']]);
+
 	sub tree_attributes { return qw(groupby aggregates) };
 	sub as_sparql {
 		my $self	= shift;
@@ -1274,7 +1312,7 @@ package Attean::Algebra::Table 0.009 {
 		my @tokens;
 		push(@tokens, $values, $lparen);
 		foreach my $var (@{ $self->variables }) {
-			push(@tokens, $var->sparql_tokens->members);
+			push(@tokens, $var->sparql_tokens->elements);
 		}
 		push(@tokens, $values, $rparen);
 		
@@ -1283,7 +1321,7 @@ package Attean::Algebra::Table 0.009 {
 			push(@tokens, $lparen);
 			foreach my $val ($row->values) {
 				# TODO: verify correct serialization of UNDEF
-				push(@tokens, $val->sparql_tokens->members);
+				push(@tokens, $val->sparql_tokens->elements);
 			}
 			push(@tokens, $rparen);
 		}
@@ -1299,8 +1337,15 @@ package Attean::Algebra::Table 0.009 {
 
 package Attean::Algebra::Ask 0.009 {
 	use Moo;
-	sub in_scope_variables { return; }
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
+	use namespace::clean;
+	
 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+	
+	sub in_scope_variables { return; }
+
 	sub algebra_as_string { return 'Ask' }
 	sub as_sparql {
 		my $self	= shift;
@@ -1321,10 +1366,17 @@ package Attean::Algebra::Ask 0.009 {
 
 package Attean::Algebra::Construct 0.009 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ArrayRef ConsumerOf);
-	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']]);
-	sub in_scope_variables { return qw(subject predicate object); }
+	use namespace::clean;
+	
 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+
+	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']]);
+
+	sub in_scope_variables { return qw(subject predicate object); }
 	sub tree_attributes { return qw(triples) };
 	sub algebra_as_string { return 'Construct' }
 	sub as_sparql {
